"use strict";

function log(msg) {
    document.getElementById("output").appendChild(document.createElement("li")).textContent = msg + "\n\n";
}

// Using this serializer instead of JSON.stringify because the latter drops undefined.
function serializeCookie(cookie) {
    let results = [];
    if ("domain" in cookie) {
        results.push(`domain: ${cookie.domain}`);
    }
    if ("firstPartyDomain" in cookie) {
        // There is a difference between the key being set, and being set to undefined.
        results.push(`firstPartyDomain: ${cookie.firstPartyDomain}`);
    }
    return "{" + results.join(", ") + "}";
}

async function runTestCases(...testcases) {
    for (let details of testcases) {
        let key = serializeCookie(details);
        try {
            let cookies = await browser.cookies.getAll(details);
            cookies = cookies.filter(c => c.domain.endsWith("example.com"));
            log(`Result for ${key} is\n${cookies.map(serializeCookie).join("\n")}`);
        } catch (e) {
            log(`Error for ${key}\n${e}`);
        }
    }
}

async function generateCookie() {
    let tab = await browser.tabs.create({ url: "https://example.com"});
    await browser.tabs.executeScript(tab.id, { code: "document.cookie = 'cookiename=cookievalue; max-age=300'; " });
    await browser.tabs.remove(tab.id);
}

async function runTests() {
    log(`Cleaning up cookies for a clean state`);
    await browser.browsingData.removeCookies({ hostnames: ["example.com"] });

    log(`Disabling FPI and setting cookie`);
    await browser.privacy.websites.firstPartyIsolate.set({value: false});
    await generateCookie();

    log(`Enabling FPI and setting cookie`);
    await browser.privacy.websites.firstPartyIsolate.set({value: true});
    await generateCookie();

    for (let value of [true, false]) {
        log(`Cookies while FPI is ${value ? "enabled": "disabled"}`);
        await browser.privacy.websites.firstPartyIsolate.set({value});
        await runTestCases({
            firstPartyDomain: null,
        }, {
            domain: "example.com",
        }, {
            domain: "example.com", firstPartyDomain: "example.com",
        }, {
            domain: "example.com", firstPartyDomain: null,
        }, {
            domain: "example.com", firstPartyDomain: undefined,
        });
    }
    log(`Finished tests. Now check the results.`);
}

runTests();
